@@ -78,20 +78,22 @@ class NameServer(object):
             raise pycom.ServiceNotFound("Interface not found: %s" %
                 interface)
 
-        if request.args.get("as_list"):
-            return [
-                {
-                    "address": record.address,
-                    "service": record.service
-                }
-                for record in records
-            ]
-
-        else:
-            return {
-                "address": records[0].address,
-                "service": records[0].service
+        service = request.args.get("service")
+        full_list = [
+            {
+                "address": record.address,
+                "service": record.service
             }
+            for record in records
+            if service is None or record.service == service
+        ]
+
+        assert len(full_list)
+
+        if request.args.get("as_list"):
+            return full_list
+        else:
+            return full_list[0]
 
     @pycom.method(constants.NS_METHOD_LIST_SERVICES)
     def method_list_services(self, request):    # pylint: disable-msg=W0613
