@@ -28,25 +28,25 @@
 
 import zmq
 
-from . import conf, constants, exceptions, interfaces, invoke, protocol
+from . import base, constants, exceptions, interfaces, protocol
 
 
 def nameserver():
-    """Get nameserver invoker."""
+    """Get nameserver component."""
     global _nameserver
 
     if _nameserver is not None:
         return _nameserver
 
-    ns_conf = conf.configuration.get("nameserver", constants.NS_NAME_LOCAL)
+    ns_conf = base.configuration.get("nameserver", constants.NS_NAME_LOCAL)
     if ns_conf == constants.NS_NAME_LOCAL:
         try:
-            ns_class = interfaces.registry[constants.IFACE_NAMESERVER]
+            ns_instance = interfaces.registry[constants.IFACE_NAMESERVER]
         except KeyError:
-            ns_class = None
+            ns_instance = None
 
-        if ns_class is not None:
-            _nameserver = invoke.DirectInvoker(ns_class())
+        if ns_instance is not None:
+            _nameserver = ns_instance.__interface__
         else:
             raise exceptions.ServiceNotFound("Nameserver not found")
     else:
@@ -55,12 +55,17 @@ def nameserver():
     return _nameserver
 
 
-def locate(iface):
-    """Locate a service providing given interface."""
-    ns_invoker = nameserver()
+def locate(iface, service_name=None):
+    """Locate a service providing given interface *iface*.
+
+    If *service_name* is specified,
+    only service with this name will be returned.
+
+    """
+    ns_component = nameserver()
     try:
-        info = ns_invoker.invoke(constants.NS_METHOD_LOCATE,
-            {"interface": iface})
+        info = ns_component.invoke(constants.NS_METHOD_LOCATE,
+            {"interface": iface, "service": service_name})
     except exceptions.RemoteError as err:
         if err.code == constants.ERROR_SERVICE_NOT_FOUND:
             raise exceptions.ServiceNotFound(
@@ -68,7 +73,7 @@ def locate(iface):
         else:
             raise
 
-    return invoke.RemoteInvoker(info["address"], iface)
+    return protocol.RemoteComponent(info["address"], iface)
 
 
 # Private
@@ -97,4 +102,4 @@ def _detect_ns(host):
         sock.close(linger=0)
         raise exceptions.ServiceNotFound("Nameserver not available")
 
-    return invoke.RemoteInvoker(sock, constants.IFACE_NAMESERVER)
+    return protocol.RemoteComponent(sock, constants.IFACE_NAMESERVER)
