@@ -26,24 +26,21 @@
 
 """JSON protocol details."""
 
-import collections
 import json
 
+from six import string_types
 import zmq
 
-from . import constants, exceptions
+from . import base, constants, exceptions
 
 
-class Request(collections.namedtuple("Request", "interface method args")):
-    """Named tuple representing request."""
-
-
-def request_to_wire(iface_name, method_name, args):
+def request_to_wire(iface_name, method_name, args, session_id=None):
     """Pack request to wire format."""
     return json.dumps({
         "version": constants.PROTO_VERSION,
         "interface": iface_name,
         "method": method_name,
+        "session_id": session_id,
         "args": args
     }).encode("utf-8")
 
@@ -51,20 +48,28 @@ def request_to_wire(iface_name, method_name, args):
 def request_from_wire(message):
     """Unpack request from wire format, return Request object."""
     message = json.loads(message.decode("utf-8"))
+
     try:
-        return Request(message["interface"], message["method"],
+        session = _session_factory.get(message["session_id"])
+    except KeyError:
+        session = _session_factory.new()
+
+    try:
+        return base.Request(message["interface"], message["method"], session,
             message.get("args", None))
     except KeyError as err:
         raise exceptions.BadRequest("Field %s is required" % err.args[0])
 
 
-def response_to_wire(code, error, result):
+def response_to_wire(code, error, result, session_id=None):
     """Pack result to wire format."""
     if code:
         message = {"code": code, "error": error}
     else:
         message = {"code": 0, "result": result}
 
+    message["session_id"] = session_id
+
     return json.dumps(message).encode("utf-8")
 
 
@@ -76,12 +81,12 @@ def response_from_wire(message):
         raise exceptions.RemoteError(message.get("error", None) or
             "Remote error #%d" % result_code, code=result_code)
 
-    return message.get("result", None)
+    return message.get("result", None), message.get("session_id", None)
 
 
-def send_request(socket, iface, method, args, timeout=None):
+def send_request(socket, iface, method, args, timeout=None, session_id=None):
     """Send request over 0MQ network and return result."""
-    message = request_to_wire(iface, method, args)
+    message = request_to_wire(iface, method, args, session_id=session_id)
     timeout = timeout or constants.PROTO_DEFAULT_TIMEOUT
 
     # Here and below we use poll() as it supports timeout
@@ -131,4 +136,34 @@ def send_request(socket, iface, method, args, timeout=None):
             raise
 
 
+class RemoteComponent(base.BaseComponent):
+    """Component working over 0MQ network."""
+
+    def __init__(self, socket, iface):
+        """C-tor. Takes 0MQ socket or address and interface name."""
+        super(RemoteComponent, self).__init__()
+        if isinstance(socket, string_types):
+            self.socket = context.socket(zmq.REQ)
+            self.socket.connect(socket)
+        else:
+            self.socket = socket
+
+        self.iface = iface
+        self.session_id = None
+
+    def invoke(self, method_name, args=None,
+            timeout=None):
+        """Invoke given method with args
+        (see :meth:`pycom.BaseComponent.invoke` for details)."""
+        result, session_id = send_request(self.socket, self.iface, method_name,
+             args, timeout=timeout, session_id=self.session_id)
+        if session_id is not None:
+            self.session_id = session_id
+        return result
+
+
 context = zmq.Context()  #: Shared 0MQ context
+
+# Private
+
+_session_factory = base.SessionFactory()
