@@ -27,23 +27,24 @@
 """Tests for pycom.apps.nameserver."""
 
 import pycom
-from pycom import constants, exceptions, invoke, protocol
+from pycom import constants, exceptions, protocol
 from pycom.apps import nameserver
-from pycom.test.utils import unittest
+
+from .utils import unittest
 
 class TestNameServer(unittest.TestCase):
 
     def setUp(self):
-        self.service = nameserver.NameServer()
-        self.invoker = invoke.DirectInvoker(self.service)
+        self.component = nameserver.NameServer.__interface__
+        self.service = self.component.instance
         self.service._services = {}
         self.service._registry = {}
 
     def testStat(self):
-        self.assertEqual(self.invoker.invoke(constants.NS_METHOD_STAT), {})
+        self.assertEqual(self.component.invoke(constants.NS_METHOD_STAT), {})
 
     def testRegisterOk(self):
-        self.assertEqual(self.invoker.invoke(constants.NS_METHOD_REGISTER,
+        self.assertEqual(self.component.invoke(constants.NS_METHOD_REGISTER,
             args={
                 "interface": "test_apps_nameserver.testRegisterOk",
                 "service": "/test_apps_nameserver/testRegisterOk",
@@ -58,13 +59,13 @@ class TestNameServer(unittest.TestCase):
             1)
 
     def testInterfaceReregisterOk(self):
-        self.assertEqual(self.invoker.invoke(constants.NS_METHOD_REGISTER,
+        self.assertEqual(self.component.invoke(constants.NS_METHOD_REGISTER,
             args={
                 "interface": "test_apps_nameserver.testReregisterOk",
                 "service": "/test_apps_nameserver/testReregisterOk",
                 "address": "null://null"
             }), {})
-        self.assertEqual(self.invoker.invoke(constants.NS_METHOD_REGISTER,
+        self.assertEqual(self.component.invoke(constants.NS_METHOD_REGISTER,
             args={
                 "interface": "test_apps_nameserver.testReregisterOk",
                 "service": "/test_apps_nameserver/testReregisterOk2",
@@ -79,7 +80,7 @@ class TestNameServer(unittest.TestCase):
             2)
 
     def testRegisterWOAddress(self):
-        self.assertRaises(exceptions.BadRequest, self.invoker.invoke,
+        self.assertRaises(exceptions.BadRequest, self.component.invoke,
             constants.NS_METHOD_REGISTER,
             args={
                 "interface": "test_apps_nameserver.testRegisterWOAddress",
@@ -92,7 +93,7 @@ class TestNameServer(unittest.TestCase):
                 "test_apps_nameserver.testLocateOk",
                 "/test_apps_nameserver/testLocateOk")
         ]
-        self.assertDictEqual(self.invoker.invoke(constants.NS_METHOD_LOCATE,
+        self.assertDictEqual(self.component.invoke(constants.NS_METHOD_LOCATE,
             args={
                 "interface": "test_apps_nameserver.testLocateOk"
             }), {
@@ -109,7 +110,7 @@ class TestNameServer(unittest.TestCase):
                 "test_apps_nameserver.testLocateList",
                 "/test_apps_nameserver/testLocateList2")
         ]
-        self.assertListEqual(self.invoker.invoke(constants.NS_METHOD_LOCATE,
+        self.assertListEqual(self.component.invoke(constants.NS_METHOD_LOCATE,
             args={
                 "interface": "test_apps_nameserver.testLocateList",
                 "as_list": True
@@ -124,18 +125,39 @@ class TestNameServer(unittest.TestCase):
                 }
             ])
 
-    def testLocateWOAddress(self):
-        self.service._registry["test_apps_nameserver.testLocateWOAddress"] = [
+    def testLocateEmpty(self):
+        self.service._registry["test_apps_nameserver.testLocateEmpty"] = [
             nameserver._Record("null://null",
-                "test_apps_nameserver.testLocateWOAddress",
-                "/test_apps_nameserver/testLocateWOAddress")
+                "test_apps_nameserver.testLocateEmpty",
+                "/test_apps_nameserver/testLocateEmpty")
         ]
-        self.assertRaises(exceptions.BadRequest, self.invoker.invoke,
+        self.assertRaises(exceptions.BadRequest, self.component.invoke,
             constants.NS_METHOD_LOCATE,
             args={})
 
+    def testLocateWithService(self):
+        self.service._registry["test_apps_nameserver.testLocateWithService"] = [
+            nameserver._Record("null://null",
+                "test_apps_nameserver.testLocateWithService",
+                "/test_apps_nameserver/testLocateWithService"),
+            nameserver._Record("null://null",
+                "test_apps_nameserver.testLocateWithService",
+                "/test_apps_nameserver/testLocateWithService2"),
+            nameserver._Record("null://null",
+                "test_apps_nameserver.testLocateWithService",
+                "/test_apps_nameserver/testLocateWithService3")
+        ]
+        self.assertDictEqual(self.component.invoke(constants.NS_METHOD_LOCATE,
+            args={
+                "interface": "test_apps_nameserver.testLocateWithService",
+                "service": "/test_apps_nameserver/testLocateWithService2",
+            }), {
+                "service": "/test_apps_nameserver/testLocateWithService2",
+                "address": "null://null"
+            })
+
     def testLocateMissing(self):
-        self.assertRaises(exceptions.ServiceNotFound, self.invoker.invoke,
+        self.assertRaises(exceptions.ServiceNotFound, self.component.invoke,
             constants.NS_METHOD_LOCATE,
             args={
                 "interface": "test_apps_nameserver.testLocateMissing"
@@ -146,7 +168,7 @@ class TestNameServer(unittest.TestCase):
             nameserver._Record("null://null",
                 "test_apps_nameserver.testListServices",
                 "/test_apps_nameserver/testListServices")
-        self.assertListEqual(self.invoker.invoke(
+        self.assertListEqual(self.component.invoke(
             constants.NS_METHOD_LIST_SERVICES,), [{
                 "service": "/test_apps_nameserver/testListServices",
                 "interface": "test_apps_nameserver.testListServices"
