@@ -32,23 +32,23 @@ import time
 
 import pycom
 import pycom.apps.nameserver
-from pycom import conf, constants, invoke, nsclient, utils
-from pycom.test.utils import unittest, Replacer
-from pycom.test.test_nsclient import BaseTest
+from pycom import base, constants, nsclient, protocol, utils
+
+from .utils import unittest, Replacer
+from .test_nsclient import BaseTest
 
 
 class MainTest(BaseTest, unittest.TestCase):
 
     def setUp(self):
         super(MainTest, self).setUp()
-        self.oldConf = conf.configuration.copy()
-        conf.configuration.clear()
-        nsclient._nameserver = invoke.DirectInvoker(
-            pycom.apps.nameserver.NameServer())
+        self.oldConf = base.configuration.copy()
+        base.configuration.clear()
+        nsclient._nameserver = pycom.apps.nameserver.NameServer().__interface__
 
     def tearDown(self):
-        conf.configuration.clear()
-        conf.configuration.update(self.oldConf)
+        base.configuration.clear()
+        base.configuration.update(self.oldConf)
         super(MainTest, self).tearDown()
 
     def testService(self):
@@ -56,8 +56,8 @@ class MainTest(BaseTest, unittest.TestCase):
         meth = "test_method"
 
         def _check(address, self):
-            invoker = invoke.RemoteInvoker(address, iface)
-            self.assertDictEqual(invoker.invoke(meth, [1, 2, 3]),
+            component = protocol.RemoteComponent(address, iface)
+            self.assertDictEqual(component.invoke(meth, [1, 2, 3]),
                 {"input": [1, 2, 3]})
 
         self._testService(iface, meth, _testMethod, _check, self)
@@ -78,10 +78,10 @@ class MainTest(BaseTest, unittest.TestCase):
 
         def _check(address, self):
             wrong_iface = iface + "/banana!"
-            invoker = invoke.RemoteInvoker(address, wrong_iface)
+            component = protocol.RemoteComponent(address, wrong_iface)
             with self.assertRaisesRegexp(pycom.RemoteError,
                     "Unsupported interface: %s" % wrong_iface) as context:
-                invoker.invoke(meth)
+                component.invoke(meth)
             self.assertEqual(context.exception.code,
                 constants.ERROR_BAD_REQUEST)
 
@@ -93,11 +93,11 @@ class MainTest(BaseTest, unittest.TestCase):
 
         def _check(address, self):
             wrong_meth = meth + "/banana!"
-            invoker = invoke.RemoteInvoker(address, iface)
+            component = protocol.RemoteComponent(address, iface)
             with self.assertRaisesRegexp(pycom.MethodNotFound,
                     "Method '%s' not found in interface %s" %
                     (wrong_meth, iface)) as context:
-                invoker.invoke(wrong_meth)
+                component.invoke(wrong_meth)
             self.assertEqual(context.exception.code,
                 constants.ERROR_METHOD_NOT_FOUND)
 
@@ -108,10 +108,10 @@ class MainTest(BaseTest, unittest.TestCase):
         meth = "test_method"
 
         def _check(address, self):
-            invoker = invoke.RemoteInvoker(address, iface)
+            component = protocol.RemoteComponent(address, iface)
             with self.assertRaisesRegexp(pycom.RemoteError,
                     "test BadRequest") as context:
-                invoker.invoke(meth)
+                component.invoke(meth)
             self.assertEqual(context.exception.code,
                 constants.ERROR_BAD_REQUEST)
 
@@ -122,15 +122,27 @@ class MainTest(BaseTest, unittest.TestCase):
         meth = "test_method"
 
         def _check(address, self):
-            invoker = invoke.RemoteInvoker(address, iface)
+            component = protocol.RemoteComponent(address, iface)
             with self.assertRaisesRegexp(pycom.RemoteError,
                     "Unknown error during request") as context:
-                invoker.invoke(meth)
+                component.invoke(meth)
             self.assertEqual(context.exception.code,
                 constants.ERROR_UNKNOWN)
 
         self._testService(iface, meth, _testMethodWithUnknownError, _check, self)
 
+    def testSession(self):
+        iface = "test.test_launcher.MainTest.testSession"
+        meth = "test_method"
+
+        def _check(address, self):
+            component = protocol.RemoteComponent(address, iface)
+            self.assertEqual(component.invoke(meth, 42), 42)
+            self.assertEqual(component.invoke(meth, 100500), 100500)
+            self.assertEqual(component.invoke(meth), 100500)
+
+        self._testService(iface, meth, _testSession, _check, self)
+
     # Private
 
     def _testService(self, iface, meth, meth_body, call, *args, **kw):
@@ -161,3 +173,9 @@ def _testMethodWithKnownError(request):
 
 def _testMethodWithUnknownError(request):
     raise TypeError("test TypeError")
+
+def _testSession(request):
+    if request.args:
+        request.session["test"] = request.args
+
+    return request.session["test"]
